@@ -70,16 +70,29 @@ test_that("Assays with NA are removed in NPX check", {
 })
 
 test_that("No error if missing data from 1st OID",{
-  expect_no_error(suppressWarnings(suppressWarnings(npx_data1.uniqIDs %>% 
+  expect_no_error(suppressWarnings(suppressWarnings(npx_data1.uniqIDs %>%
     mutate(QC_Warning = ifelse(OlinkID == "OID01216" & stringr::str_detect(SampleID, "2"),
                                "Warn", QC_Warning)) %>%
-    filter(QC_Warning == "Pass") %>% 
+    filter(QC_Warning == "Pass") %>%
     olink_pca_plot())))
-  expect_no_error(suppressWarnings(suppressWarnings(npx_data1.uniqIDs %>% 
+  expect_no_error(suppressWarnings(suppressWarnings(npx_data1.uniqIDs %>%
                                                       mutate(QC_Warning = ifelse(OlinkID == "OID01216" & stringr::str_detect(SampleID, "A"),
                                                                                  "Warn", QC_Warning)) %>%
-                                                      filter(QC_Warning == "Pass") %>% 
+                                                      filter(QC_Warning == "Pass") %>%
                                                       olink_pca_plot())))
-  
-  
+
+
+})
+
+# data with SampleQC instead of QC_Warning
+w2 <- testthat::capture_error(
+  procData_missingData <- npx_data1.uniqIDs %>% dplyr::rename(SampleQC = QC_Warning) %>%
+    npxProcessing_forDimRed(color_g = 'QC_Warning',
+                            drop_assays = F,
+                            drop_samples = F,
+                            verbose = T)
+  )
+
+test_that("npxProcessing_forDimRed does not recognize QC_Warning", {
+  expect_equal(w2, simpleError("In color_g = \"QC_Warning\", QC_Warning was not found. Did you mean color_g = \"SampleQC\"?"))
 })
